;+
;pro sage_filenames,versspec,versindex,compspec,compindex,$
;                 version=version,compare_version=compare_version,yyyymm=yyyymm
;Subroutine of sageprof for getting the filenames.
;There are multiple ways of specifying the filenames for sageprof;
;this subroutine analyzes user input to determine the correct 
;filenames.
;-
;note: sage_filenames is the last program in this file.  There are
;also a number of more specific subroutines...

function method1,defaultname,defaultpath
;METHOD 1:
;The easiest input mode to analyze is when the user selects the
;file using the browse dialog box.  This occurs when the 
;keyword is set to the value "1" (or set with the slash notation).
;
;the output is the filename from the browse dialog box
;DEFAULTNAME is the name of the system variable which holds
;the default filename
;
;if this is not the first time in the session that the dialog box
;is used, there will be a default filename
defsysv,defaultname,exists=exists
if exists ne 0 then begin
    if defaultname eq '!_versdefault' then begin
        file = !_versdefault
        path = !_verspathdef
    endif else if defaultname eq '!_compdefault' then begin
        file = !_compdefault
        path = !_comppathdef
    endif
    specname = dialog_pickfile(/must_exist,$
                               title='Select file to read',$
                               file=file,path=path,$
                               filter='*SPEC*')
endif else begin
    specname = dialog_pickfile(/must_exist,$
                               title='Select file to read',$
                               filter='*SPEC*')
endelse
if specname eq '' then retall
;set the selected file to be the default file for next time
defsysv,defaultname,specname
;strip out the path name to set the default path
rev = string(reverse(byte(specname)))
pos = strpos(rev,'\') > strpos(rev,'/')
defsysv,defaultpath,$
  string(reverse(byte(strmid(rev,pos,strlen(rev)-1))))        

return,specname
end

function method3,version,shortcuts
match = where(strupcase(version) eq strupcase(shortcuts.version))
if match(0) eq -1 then begin
;the user has specified a shortcut that doesn't exist.  
;It could be the full path and filename rather than a shortcut...
    return,0
endif else if n_elements(match) gt 1 then begin
;the shortcut file contains identical keys.  Error...
    print,'SAGEPROF ERROR: the configuration file contains '+$
      'multiple identical keys'
    usage
    return,0
endif else begin
    return,shortcuts(match(0)).path
endelse 
end

function getindexname,specname,filetype=filetype
;construct the index file name and read that file
indexfile = specname
p = strpos(strupcase(indexfile),'SAGE_I')
p = strpos(indexfile,'_',p+6)+1
q = strpos(indexfile,'_',p)
filetype = strmid(indexfile,p,q-p)
l = q-p
p = strpos(indexfile,filetype)
while p ne -1 do begin
    q = strlen(indexfile)
    m = p+l
    indexfile = strmid(indexfile,0,p)+'INDEX'+strmid(indexfile,m,q-m)
    p = strpos(indexfile,filetype,m)
endwhile
return,indexfile
end

pro parsedate,filename,yyyymm,replace=replace
months = ['january','february','march','april','may','june',$
          'july','august','september','october','november','december']
;parse a standard structure filename for the date
p = strpos(filename,'SAGE_I')
q = strpos(filename,'_',p+10)
date = long(strmid(filename,q+1,6))

if not keyword_set(replace) then begin
;mode 1; change yyyymm to reflect the filename
    yyyymm = date
endif else begin
;mode 2; replace the filename using the given date
    if n_elements(yyyymm) eq 1 then begin
;before replacing the filename, strip out the year and month
        cyyyy = strtrim(fix(date/100L),2)
        mm = date mod 100
;replace the filename
        strput,filename,strtrim(yyyymm,2),q+1
;also check the path name for year and spelled-out month
        r = strpos(filename,cyyyy)
;if the first instance is before the one in the base name, replace
        if r gt -1 and r lt q+1 then begin
;get the new yyyy
            cyyyy = strtrim(fix(yyyymm/100L),2)
            strput,filename,cyyyy,r
        endif
;and replace the spelled out month name, if necessary
        r = strpos(filename,months(mm-1))
        if r ne -1 then begin
            head = strmid(filename,0,r)
            mlen = strlen(months(mm-1))
            flen = strlen(filename)
            tail = strmid(filename,r+mlen,flen-mlen-r)
;get the new mm
            mm = yyyymm mod 100
            filename = head+months(mm-1)+tail
        endif
    endif
endelse
end

pro sage_filenames,versspec,versindex,compspec,compindex,filetype,$
        version=version,compare_version=compare_version,yyyymm=yyyymm,$
        testing=testing

;initialize the output
versspec = 0 & versindex = 0 & compspec = 0 & compindex = 0
;the following variable keep track of whether the file was
;specified directly by the user
directvers = 0 & directcomp = 0

;the user may specify the file using a browse dialog box,
;which is triggered by setting the version keyword (or 
;compare_version keyword) with the slash notation or 
;setting it equal to 1.

if keyword_set(version) then begin
    if strtrim(version,2) eq '1' then begin
        versspec = method1('!_versdefault','!_verspathdef')
        directvers = 1
    endif
endif
;repeat for the comparison version
if keyword_set(compare_version) then begin
    if strtrim(compare_version,2) eq '1' then begin
        compspec = method1('!_compdefault','!_comppathdef')
        directcomp = 1
    endif
endif

;keep going? Check whether the filenames are still zero
if (not keyword_set(versspec) or not keyword_set(compspec)) then begin
;SHORTCUT FILE...
;The user might choose to specify a filepath shortcut
    setshortcuts,shortcuts
;get the type code for the result
    sz = size(shortcuts) & type = sz(n_elements(sz)-2)
;if there was no configuration file, the result is not a structure...
    if type eq 8 then shorts = 1 else shorts = 0
;METHOD 2: use the default
    if (not keyword_set(versspec)) and not keyword_set(version) then begin
;the default is in the shortcuts array
        if shorts eq 1 then begin
            last = n_elements(shortcuts) - 1
            versspec = shortcuts(last).path
        endif else begin
;if there was no shortcut file, this is an error
            usage
        endelse
    endif
;...there is no default compare version.  It's not needed.
endif

;keep going?
if not keyword_set(versspec) then begin
    if shorts eq 1 then begin
;METHOD 3: use a shortcut name from the configuration file
        temp = method3(version,shortcuts)
;check that temp is not equal to zero
        if keyword_set(temp) then versspec = temp
    endif
endif
;repeat for compare version
if not keyword_set(compspec) and keyword_set(compare_version) then begin
    if shorts eq 1 then begin
        temp = method3(compare_version,shortcuts)
        if keyword_set(temp) then compspec = temp
    endif
endif

;keep going?
if not keyword_set(versspec) then begin
;METHOD 4:
;the user may specify the full filename
    res = findfile(version,count=count)
;or the full filename except for an extension
;(this is in case an extension or trailing character is added 
;to the base file name as on the version 6.00 CD)
    if count eq 0 then begin
        res = findfile(version+'*',count=count)
    endif
    if count ne 1 then begin
;if testing, let it go; assume all given filenames are good
        if keyword_set(testing) then begin
            versspec = version
            directvers = 1
        endif else begin
;otherwise print an error
            print,'SAGEPROF ERROR: could not find specified filename'
            print,'     '+version
            usage
        endelse
    endif else begin
        versspec = res(0)
        directvers = 1
    endelse
endif
;repeat for compare version
if (not keyword_set(compspec) and keyword_set(compare_version)) then begin
;the user may specify the full filename
    res = findfile(compare_version,count=count)
;or the full filename except for an extension
;(this is in case an extension or trailing character is added 
;to the base file name as on the version 6.00 CD)
    if count eq 0 then begin
        res = findfile(compare_version+'*',count=count)
    endif
    if count ne 1 then begin
;if testing, assume all given filenames are okay
        if keyword_set(testing) then begin
            compspec = compare_version
            directcomp = 1
        endif else begin
;otherwise print an error
            print,'SAGEPROF ERROR: could not find specified filename'
            print,'     '+compare_version
            usage
        endelse
    endif else begin
        compspec = res(0)
        directcomp = 1
    endelse
endif

parsedate,versspec,vdate
if keyword_set(compare_version) then begin
    parsedate,compspec,cdate
endif

;if the date keyword is not set, set it using the filename
if not keyword_set(yyyymm) then begin
    if (directvers eq 1) then begin
        yyyymm = vdate
    endif else if (directcomp eq 1) then begin
        yyyymm = cdate
    endif else begin
;still don't have the date?  print an error
        print,'DATE must be specified'
        usage
    endelse   
endif

;might need to change the date in the filenames, if they 
;were generated by default, or if the compare file doesn't
;match the version file
if vdate ne yyyymm then begin
    parsedate,versspec,yyyymm,/replace
endif
if keyword_set(compare_version) then begin
    if cdate ne yyyymm then begin
        parsedate,compspec,yyyymm,/replace
    endif
endif

;Finally, get the matching index files
versindex = getindexname(versspec,filetype=filetype)
if keyword_set(compare_version) then compindex = getindexname(compspec)

end






