;+
;This program can be used to survey SAGE II SPEC files.  Other files
;that should be in the directory are included in the file
;sageprof.tar.  They are: bit.pro, bitinfo.pro, bitinfo_610.pro,
;click.pro, ephinfo_y2k.pro, getstructinfo.pro, indexinfo_60d.pro,
;indexinfo_610.pro, meanprof.pro, medianprof.pro, metinfo.pro,
;metinfo_610.pro, plotinfo.pro, printbits.pro, readstructs.pro,
;refractinfo_60d.pro, sage_filenames.pro, sagebits.pro,
;setshortcuts.pro, specinfo_600.pro, specinfo_610.pro,
;transinfo_600.pro,
;
;BASIC USAGE:
;Type "sageprof" for usage examples.  At the minimum, the month
;or date must be specified using the DATE keyword, and the
;species of interest must also be given, using the SPECIES keyword.  
;Possible species are 'O3','NO2','H2O','Density','Ext1020','Ext525',
;'Ext453','Ext385','Dens_mid_atm','SurfDen' and 'Radius'.
;Also the version or pathname to the data must be provided either 
;using a configuration file or the VERSION keyword (both described below).
;
;The events to be displayed can be limited using the LAT and DLAT
;keywords to provide a latitude range (dlat = delta latitude).
;Other optional keywords for limiting events are TYPE (valid values
;are 'sr' and 'ss) and EVENT (the event number; useful if attempting 
;to plot one particular event)
;
;CHOOSING THE INPUT FILES:
;There are multiple ways to choose the file to examine.
;If you are going to use sageprof a lot, it is a good idea to set
;up a configuration file called sageprof.conf which will contain 
;version shortcuts.  This file should be in the local directory
;or alternately in the directory containing the source file
;sageprof.pro, if that is a different directory.  The file 
;sageprof.conf is a text file, with pairs of lines 
;giving (1) the version shortcut, and (2) the full path and filename of any 
;given species file for that version (the date part of the filename
;is not significant).  No blank lines should be 
;included.  The following sample includes two defined versions:
;v6.00
;/g2prof1/v6.00/SAGE_II_SPEC_198410.6.00
;v6.10
;/snappy2/v6.10/SAGE_II_SPEC_198410.6.10
;
;With the sageprof.conf file in place, the VERSION keyword and 
;COMPARE_VERSION keyword (if desired; see below for description)
;should be set with a string containing the shortcut for the 
;desired version.  If the version keyword is not set, the default
;is the last entry in the configuration file.
;
;Alternately, the user can supply the full filename in one of 
;two ways.  Either set the VERSION keyword (and/or COMPARE_VERSION 
;if desired) equal to the path and file name(s) of the desired
;files, or set the keywords using the slash notation.  If the
;slash notation is used, a browse dialog box will pop up and the
;file can be specified that way.  A separate browse dialog box
;appears if the COMPARE_VERSION keyword is set this way.
;
;(Note, as a convenience to the user, if either the VERSION or
;COMPARE_VERSION keywords are supplied with the full filename,
;in one of the two ways described above, the DATE keyword does
;not need to be set.  Sageprof will extract the date from the
;filename.)
;
;COMPARISON BETWEEN TWO VERSIONS
;The COMPARE_VERSION keyword can be set to compare profiles from two
;different versions.  It can be set in any of the ways described
;above for the VERSION keyword.  The data from the two versions will be
;plotted together on the same plot, with the "compare_version" data 
;done in color.  Alternately, the /PDIF keyword can be used to plot
;percent differences, instead of overlaid profiles.
;
;DESCRIPTION OF OTHER KEYWORDS AND OPTIONS:
;This program plots the profiles that match all the specifications to
;the current device one at a time, or all in the same frame if the
;/ALL keyword is also set.  If it is plotting to the screen, the
;program will wait for the left mouse button to be depressed between
;plots.  Depressing the right mouse button will end the program early
;if necessary.  The keyword /MEAN can be used to plot a mean profile
;of all selected events, or the keyword /MEDIAN can be used to plot
;a median profile.  If MEAN or MEDIAN is used in conjunction with
;PDIF (see above), the percent difference calculation is done last.
;
;The BITSHOW keyword is provided for examining the bit flags which
;are included in the SAGE II index files.  If this keyword is set,
;and mean, median, all and pdif are not set (so that each event
;plots individually), a list of bit flags will be shown underneath 
;the plot's x-axis.  These are the bit flags that are "on" for that
;event.  If used in conjuction with COMPARE_VERSION, the flag names
;will be color coded using the same colors as the plotted data.
;
;Data can also be filtered by error bar, using
;the THRESHOLD keyword.  THRESHOLD should be set to the maximum 
;allowable error bar, in percent.
;
;If the OUTPUT keyword is set, the data will also be written to an
;output file.  The name of the file can be given by the OUTPUT
;keyword, or if no filename is given, it is 'sageprof.dat'
;
;PLOTTING KEYWORDS
;Most of the usual IDL plotting keywords can be used to override the
;default plotting options.  Examples include XRANGE,YRANGE,LINESTYLE
;and PSYM.
;
;Special keyword are also included to change the look of other lines
;on the plot:
;ULINESTYLE, UCOLOR, UTHICK, UPSYM, USYMSIZE: sets characteristics of
;the uncertainty lines that are shown along with the data
;CCOLOR, CLINESTYLE, CTHICK, CPSYM, CSYMSIZE: sets characteristics of
;the line used to show comparison data when COMPARE_VERSION is used
;CUCOLOR, CULINESTYLE, CUTHICK, CUPSYM, CUSYMSIZE: for the
;uncertainties attached to the comparison data.
;
;Note that all keywords can be shortened, just as in any IDL
;procedure.  Examples: VERS instead of VERSION, COMP instead of
;COMPARE_VERSION, BITS instead of BITSHOW, LINES instead of LINESTYLE 
;
;$Header: /home/users/burton/sageprof-rcs/sageprof.pro,v 1.63 2003/10/01 15:57:17 burton Exp $
;-

pro usage
print,'usage examples:'
print,'sageprof,date=19841024,spec=''o3''   ;*'
print,'sageprof,vers=''v6.00'',date=199903,spec=''h2o'',type=''SR'',/bits   ;*'
print,'sageprof,spec=''ext1020'',date=198808,lat=50,dlat=10,/all   ;*'
print,'sageprof,spec=''O3'',vers=''/localtestdirectory/SAGE_II_SPEC_199609.6.10'''
print,'sageprof,spec=''O3'',comp=''/localtestdirectory/SAGE_II_SPEC_199609.6.10''   ;*'
print,'sageprof,vers=''v6.00'',date=200012,spec=''ext1020'',lat=45,dlat=5,/mean   ;*'
print,'sageprof,date=200012,spec=''ext1020'',/median   ;*'
print,'sageprof,date=199603,spec=''o3'',thresh=50   ;*'
print,'sageprof,spec=''ext1020'',/version    ;(provides browse dialog box)'
print
print,'* NOTE: examples marked with * assume the presense of a sageprof.conf file'
print,'See program listing for more details, or call "doc_library,''sageprof''"'
retall
end

pro sageprof,species=species,date=date,type=type,lat=lat,dlat=dlat,$
             version=version,event=event,threshold=threshold,$
             trans=trans,all=all,output=output,$
             compare_version=compare_version,$
             pdif=pdif,absdif=absdif,mean=mean,median=median,$
             lon=lon,dlon=dlon,$
             xlog=xlog,_EXTRA=extrakeys,$
             color=color,linestyle=linestyle,thick=thick,$
             psym=psym,symsize=symsize,$
             ucolor=ucolor,ulinestyle=ulinestyle,uthick=uthick,$
             upsym=upsym,usymsize=usymsize,$
             ccolor=ccolor,clinestyle=clinestyle,cthick=cthick,$
             cpsym=cpsym,csymsize=csymsize,$
             cucolor=cucolor,culinestyle=culinestyle,cuthick=cuthick,$
             cupsym=cupsym,cusymsize=cusymsize,nouncertainty=nouncertainty,$
             bitshow=bitshow,error_return=error_return,noclick=noclick

;----------------------------------------------------------------------

error_return = 0

;check for conflicting keywords
if keyword_set(median) and keyword_set(mean) then begin
    print,'Must not use both MEAN and MEDIAN keywords'
    usage
endif
;set nouncertainty if median is set
if keyword_set(median) then begin
    nouncertainty = 1
endif

;translate inputs into useful info
;SPEC
if not keyword_set(species) then begin
    print,'SPECIES keyword must be specified'
    usage
endif

;TYPE
types = ['SR','SS']
if keyword_set(type) then begin
    type = strupcase(type)
    if (type ne 'SR' and type ne 'SS') then begin
        print,'TYPE not recognized: ',type
        usage
    endif
endif

;LAT and LON
if keyword_set(lat) and not keyword_set(dlat) then begin
    print,'both LAT and DLAT must be specified, or neither'
endif else if not keyword_set(lat) and not keyword_set(dlat) then begin
    lat = 0.
    dlat = 90.
endif
if keyword_set(lon) and not keyword_set(dlon) then begin
    print,'both LON and DLON must be specified, or neither'
endif else if not keyword_set(lon) and not keyword_set(dlon) then begin
    lon = 0.
    dlon = 180.
endif

;TRANS
if keyword_set(trans) then begin
    p = strpos(strupcase(species),'SPOD')
    if p eq -1 then begin
        print,'TRANS keyword not available for the specified species'
        print,'keyword.  It is for converting optical depth to transmission.'
        retall
    endif
endif

;OUTPUT
if keyword_set(output) then begin
    if strtrim(output,2) eq '1' then begin
        output = 'sageprof.dat'
    endif
    openw,outlun,output,/get_lun
endif

;DATE
if keyword_set(date) then begin
    len = strlen(strtrim(date,2))
    if (len ne 8 and len ne 6) or (len eq 6 and date/100L lt 1979) then begin
        print,'DATE not recognized: ',date
        usage
    endif
    if (len eq 6) then begin
        yyyymm = date
        yyyymmdd = 0
    endif else begin
        yyyymm = long(date/100L)
        yyyymmdd = date
    endelse
endif else begin
    yyyymm = 0
    yyyymmdd = 0
endelse

;VERSION and COMPARE_VERSION: 
sage_filenames,filename,indexfile,filename2,indexfile2,filetype,$
  version=version,compare_version=compare_version,yyyymm=yyyymm

if (keyword_set(pdif) or keyword_set(absdif)) and not keyword_set(compare_version) then begin
    print,'PDIF keyword is only appropriate when two versions are specified.'
    retall
endif

;change the cursor to an hourglass
cm = ['0'X,'0'X,'3FFC'X,'1008'X, '0E10'X,'07E0'X,'03C0'X,'0180'X, $
      '0180'X,'0240'X,'0420'X,'0910'X,'1188'X,'3FFC'X,'0'X,'0'X]
ci = 'FFFF'X-cm
if !d.name ne 'PS' then device,cursor_image=ci,cursor_mask=cm
;read in the structure file
readstructs,filename,spec,error_return=err
error_return=err
;if the file wasn't found, try adding an extension
if (err ne 0) then begin
    print,'error from readstructs.  returning...'
    error_return = err
    retall
endif
if !d.name ne 'PS' then device,/cursor_crosshair

;read the index file
if !d.name ne 'PS' then device,cursor_image=ci,cursor_mask=cm
readstructs,indexfile,index,exten_ret=vexten
if !d.name ne 'PS' then device,/cursor_crosshair

;get the fill value
fill = index.fillval

;read comparison files
if keyword_set(compare_version) then begin
    if !d.name ne 'PS' then device,cursor_image=ci,cursor_mask=cm
    readstructs,indexfile2,index2,exten_ret=vexten2
    readstructs,filename2,spec2
    if !d.name ne 'PS' then device,/cursor_crosshair
endif

;find the events that match the specifications
if (yyyymmdd ne 0) then begin
    ix = where(index.yyyymmdd eq yyyymmdd)
endif else begin
    ix = where(index.yyyymmdd gt yyyymm*100L and $
                index.yyyymmdd lt (yyyymm+1L)*100L)
endelse
if keyword_set(event) and ix(0) ne -1 then begin
    evix = where(index.event_num(ix) eq event)
    if evix(0) ne -1 then ix = ix(evix) else ix = -1
endif
if keyword_set(type) and ix(0) ne -1 then begin
    srss = where(types eq type) & srss = srss(0)
    six = where(index.type_sat(ix) eq srss)
    if six(0) ne -1 then ix = ix(six) else ix = -1
endif
if (ix(0) ne -1) then begin
    latix = where( abs(index.lat(ix)-lat) le dlat)
    if (latix(0) ne -1) then ix = ix(latix) else ix = -1
endif
if (ix(0) ne -1) then begin
    diflon = acos(cos( (index.lon(ix)-lon)*!dtor ))/!dtor
    lonix = where(diflon le dlon)
    if (lonix(0) ne -1) then ix = ix(lonix) else ix = -1
endif
;filter out dropped events
if (ix(0) ne -1) then begin
    ndrp = where(index.dropped(ix) eq 0,nev)
    if nev ne 0 then ix = ix(ndrp) else ix = -1
endif
if (ix(0) eq -1) then begin
    print,'No events found matching specifications'
    error_return=99
    return
endif

;pick out the structure element that is the species of interest
tgnm = tag_names(spec)
ixsp = where(tgnm eq strupcase(species)) & ixsp = ixsp(0)
if ixsp eq -1 then begin
    print,'SPECIES keyword not recognized'
    help,spec,/struct
    return
endif

;pick out the element in the plotstructure
plotinfo,plotstr
pix = where(plotstr.name eq strupcase(species)) & pix = pix(0)
if pix eq -1 then begin
    pix = 0
    print,'WARNING: using default plot options'
endif

;find the error bars
errorname = strupcase(species+'_err')
ixer = where(tgnm eq errorname) & ixer = ixer(0)

;repeat for the comparison file
if keyword_set(compare_version) then begin
    tgnm2 = tag_names(spec2)
    ixsp2 = where(tgnm2 eq strupcase(species)) & ixsp2 = ixsp2(0)
    if ixsp2 eq -1 then begin
        print,'SPECIES keyword not recognized for comp structure'
        help,spec2,/struct
        return
    endif
    ixer2 = where(tgnm2 eq errorname) & ixer2 = ixer2(0)
endif

;set up colors
if !d.name ne 'PS' and !d.window eq -1 then window
white = !d.n_colors-1L
black = 0
tvlct,255,255,255,white
tvlct,0,0,0,black
if !d.name eq 'X' or !d.name eq 'Win' then baseclr = white else baseclr = black
contrast = white - 1L
tvlct,255,0,255,contrast

;options for the line appearance
if not keyword_set(color) then color = baseclr
if not keyword_set(ucolor) then ucolor = color
if not keyword_set(ulinestyle) then ulinestyle = 2
if not keyword_set(ccolor) then ccolor = contrast
if not keyword_set(cucolor) then cucolor = ccolor
if not keyword_set(culinestyle) then culinestyle = ulinestyle
if keyword_set(thick) then begin
    if not keyword_set(uthick) then uthick = thick
    if not keyword_set(cthick) then cthick = thick
    if not keyword_set(cuthick) then cuthick = uthick
endif

;plot the axes
if keyword_set(all) or keyword_set(mean) or keyword_set(median) then begin
    if yyyymmdd ne 0 then begin
        title=string(yyyymmdd,form='(i8)') 
    endif else begin
        title=string(yyyymm,form='(i6)')
    endelse
    if keyword_set(type) or keyword_set(dlat) then begin
        minlat = min(index.lat(ix),max=maxlat)
        title = title + '  ' + string(minlat,form='(f5.1)') + $
          ' to ' + string(maxlat,form='(f5.1)') + ' lat'
    endif
    if keyword_set(type) then title = title + '  ' + type

    if keyword_set(pdif) then begin
        xrg = [-100,100] 
        xlg = 0
    endif else if keyword_set(absdif) then begin
        xrg = [-1,1]*plotstr(pix).xrange(1)
        xlog = 0
    endif else begin
        xrg = plotstr(pix).xrange
        if n_elements(xlog) eq 0 then xlg = plotstr(pix).log else xlg = xlog
    endelse

    plot,/nodata,plotstr(pix).xrange,xrange=xrg,/xstyle,$
      plotstr(pix).yrange,yrange=plotstr(pix).yrange,/ystyle,$
      xlog=xlg,_EXTRA=extrakeys,$
      ytitle='Altitude (km)',xtitle=species,title=title,$
      color=color,linestyle=linestyle,thick=thick,psym=psym,symsize=symsize
endif

;plot the data
alt = findgen(200)*0.5+0.5
;for experimental density, it's a different altitude range
if strpos(strupcase(species),'MID_ATM') ne -1 or $
 (strpos(strupcase(filename),'EXP') ne -1 and $
  strpos(strupcase(species),'EXP') ne -1) then begin
    alt = index.alt_mid_atm
endif

;calculate the mean profile, if requested
if keyword_set(mean) or keyword_set(median) then begin
    alt = alt(0:n_elements(spec(0).(ixsp))-1)
    target = alt
    data = spec(ix).(ixsp)
    if n_elements(threshold) gt 0 then begin
        bd = where(spec(ix).(ixer)/100. gt threshold or $
                   spec(ix).(ixsp) eq fill)
        if bd(0) ne -1 then begin
            data(bd) = fill
        endif
    endif
    if keyword_set(trans) then begin
        gd = where(data ne fill)
        if gd(0) ne -1 then data = exp(-data)
    endif
    if keyword_set(mean) then begin
        meanprof,alt,data,target,meanprof,count,stddev
    endif else begin ;for median
        medianprof,alt,data,target,meanprof,count
    endelse
    gd = where(count gt 0)
    if not keyword_set(pdif) and not keyword_set(absdif) then begin
        oplot,[meanprof(gd)],[target(gd)],$
          color=color,linestyle=linestyle,thick=thick,psym=psym,symsize=symsize
        if not keyword_set(nouncertainty) then begin
            oplot,[meanprof(gd)+stddev(gd)],[target(gd)],$
              color=ucolor,linestyle=ulinestyle,thick=uthick,psym=upsym,symsize=usymsize
            oplot,[meanprof(gd)-stddev(gd)],[target(gd)],$
              color=ucolor,linestyle=ulinestyle,thick=uthick,psym=upsym,symsize=usymsize
        endif
    endif
    if keyword_set(compare_version) then begin
        data2 = spec2(ix).(ixsp2)
;if threshold is set, filter out data with large error bars
        if n_elements(threshold) gt 0 then begin
            bd = where(spec2(ix).(ixer2)/100. gt threshold)
            if bd(0) ne -1 then begin
                data2(bd) = fill
            endif
        endif
        if keyword_set(trans) then begin
            gd = where(data2 ne fill)
            if gd(0) ne -1 then data2 = exp(-data2)
        endif
        if keyword_set(mean) then begin
            meanprof,alt,data2,target,mean2,count2,stddev2
        endif else begin ;for median
            medianprof,alt,data2,target,mean2,count2
        endelse
        if keyword_set(pdif) then begin
;plot the percent difference of the mean profiles
            gd = where(count gt 0 and count2 gt 0)
            oplot,[(mean2(gd)/meanprof(gd)-1.)*100.],[target(gd)],$
              color=color,linestyle=linestyle,thick=thick,psym=psym,symsize=symsize
            oplot,[0,0],plotstr(pix).yrange,line=1,$
              color=color,thick=thick
        endif else if keyword_set(absdif) then begin
            gd = where(count gt 0 and count2 gt 0)
            oplot,[mean2(gd)-meanprof(gd)],[target(gd)],$
              color=color,linestyle=linestyle,thick=thick,psym=psym,symsize=symsize
            oplot,[0,0],plotstr(pix).yrange,line=1,$
              color=color,thick=thick
        endif else begin
            gd2 = where(count2 gt 0)
            oplot,[mean2(gd2)],[target(gd2)],$
              color=ccolor,linestyle=clinestyle,thick=cthick,$
              psym=cpsym,symsize=csymsize
            if not keyword_set(nouncertainty) then begin
                oplot,[mean2(gd2)+stddev2(gd2)],[target(gd2)],$
                  color=cucolor,linestyle=culinestyle,thick=cuthick,$
                  psym=cupsym,symsize=cusymsize
                oplot,[mean2(gd2)-stddev2(gd2)],[target(gd2)],$
                  color=cucolor,linestyle=culinestyle,thick=cuthick,$
                  psym=cupsym,symsize=cusymsize
            endif
        endelse
    endif
    if keyword_set(output) then begin
        printf,outlun,species
        inrg = where(alt ge !y.crange(0) and $
                     alt le !y.crange(1), ninrg)
        for ialt = 0,ninrg-1 do begin
            printf,outlun,alt(gd(inrg(ialt))),data(inrg(ialt))
        endfor
        printf,outlun
        close,outlun & free_lun,outlun
    endif
    print,'Finished'
    return
endif
;otherwise, cycle through each profile
for i = 0,nev-1 do begin
    iev = ix(i)
;filter out fill and big error bars, if required
    if n_elements(threshold) gt 0 then begin
        gd = where(spec(iev).(ixsp) ne fill and $
         spec(iev).(ixer)/100. le threshold)
    endif else begin
        gd = where(spec(iev).(ixsp) ne fill)
    endelse
    if keyword_set(pdif) then begin
        if n_elements(threshold) gt 0 then begin
            gd = where(spec(iev).(ixsp) ne fill and $
                       spec2(iev).(ixsp2) ne fill and $
                       spec(iev).(ixer)/100. le threshold and $
                       spec2(iev).(ixer2)/100. le threshold)
        endif else begin
            gd = where(spec(iev).(ixsp) ne fill and spec2(iev).(ixsp2) ne fill)
        endelse
        if index2.dropped(iev) ne 0 then gd = -1
    endif else if keyword_set(compare_version) then begin
        if n_elements(threshold) gt 0 then begin
            gd2 = where(spec2(iev).(ixer2)/100. le threshold)
        endif else begin
            gd2 = where(spec2(iev).(ixsp2) ne fill)
        endelse
        if index2.dropped(iev) ne 0 then gd2 = -1
    endif
    if gd(0) ne -1 then begin
        data = spec(iev).(ixsp)(gd)
        if keyword_set(trans) then data = exp(-data)
        if keyword_set(pdif) then begin
            data = (spec2(iev).(ixsp2)(gd)/data - 1.) * 100.
        endif else if keyword_set(absdif) then begin
            data = spec2(iev).(ixsp2)(gd) - data
        endif else if keyword_set(compare_version) then begin
            data2 = spec2(iev).(ixsp2)(gd2)
            if keyword_set(trans) then data2 = exp(-data2)
        endif
        if not keyword_set(all) then begin
            if keyword_set(pdif) then begin
                xrg = [-100,100] 
                xlg = 0
            endif else if keyword_set(absdif) then begin
                xlg = 0
                xrg = [-1,1]*plotstr(pix).xrange(1)
            endif else begin
                xrg = plotstr(pix).xrange
                if n_elements(xlog) eq 0 then begin
                    xlg = plotstr(pix).log
                endif else begin
                    xlg = xlog
                endelse
            endelse
            if keyword_set(bitshow) then begin
                ymargin = !y.margin + [5,0]
            endif else begin 
                ymargin = !y.margin 
            endelse
            plot,[data],[alt(gd)],$
              title=string(index.yyyymmdd(iev),form='(i8)')+$
              '_'+strtrim(index.event_num(iev),2)+'  '+$
              string(index.lat(iev),form='(f5.1)')+'  '+$
              string(index.lon(iev),form='(f6.1)')+'  '+$
              types(index.type_sat(iev)),$
              xtitle=species,xrange=xrg,/xstyle,$
              ytitle='Altitude (km)',yrange=plotstr(pix).yrange,/ystyle,$
              xlog=xlg,_EXTRA=extrakeys,$
              color=color,linestyle=linestyle,thick=thick,$
              psym=psym,symsize=symsize,ymargin=ymargin
;plot error bars if available
            if ixer ne -1 and not (keyword_set(pdif) or keyword_set(absdif))then begin 
;convert the error bars from percent * 100
                ebar = spec(iev).(ixer)*1.e-4*spec(iev).(ixsp)
                if keyword_set(trans) then ebar = ebar*data
                if not keyword_set(nouncertainty) then begin
                    oplot,[data+ebar(gd)],[alt(gd)],$
                      color=ucolor,linestyle=ulinestyle,thick=uthick,$
                      psym=upsym,symsize=usymsize
                    oplot,[data-ebar(gd)],[alt(gd)],$
                      color=ucolor,linestyle=ulinestyle,thick=uthick,$
                      psym=upsym,symsize=usymsize
                endif
;if it is transmission or optical depth data, also show the standard deviation
;also check that the version is at least as recent as 6.0D
                if filetype eq 'TRANS' and $
                  strpos(strupcase(species),'SPOD') ne -1 and $
                   strpos(filename,'6.0A') eq -1 and $
                   strpos(filename,'6.0B') eq -1 and $
                   strpos(filename,'6.0C') eq -1 then begin
                    ebar = ebar*sqrt(spec(iev).npoints)
                    if not keyword_set(nouncertainty) then begin
                        oplot,[data+ebar(gd)],[alt(gd)],$
                          color=ucolor,linestyle=ulinestyle,thick=uthick,$
                          psym=upsym,symsize=usymsize
                        oplot,[data-ebar(gd)],[alt(gd)],$
                          color=ucolor,linestyle=ulinestyle,thick=uthick,$
                          psym=upsym,symsize=usymsize
                    endif
                endif
            endif
;print out the bitinfo data
            if keyword_set(bitshow) then printbits,index,iev,vexten,color
;overplot comparison data
            if keyword_set(compare_version) and $
              not keyword_set(pdif) and not keyword_set(absdif) then begin
                oplot,[data2],[alt(gd2)],$
                  color=ccolor,linestyle=clinestyle,thick=cthick,$
                  psym=cpsym,symsize=csymsize
                if ixer ne -1 then begin
                    ebar = spec2(iev).(ixer2)*1.e-4*spec2(iev).(ixsp2)
                    if keyword_set(trans) then ebar = ebar*data2
                    if not keyword_set(nouncertainty) then begin
                        oplot,[data2+ebar(gd2)],[alt(gd2)],$
                          color=cucolor,linestyle=culinestyle,thick=cuthick,$
                          psym=cupsym,symsize=cusymsize
                        oplot,[data2-ebar(gd2)],[alt(gd2)],$
                          color=cucolor,linestyle=culinestyle,thick=cuthick,$
                          psym=cupsym,symsize=cusymsize
                    endif
                endif
                if keyword_set(bitshow) then begin
                    printbits,index2,iev,vexten2,ccolor,/rhs
                endif
            endif
            if keyword_set(pdif) or keyword_set(absdif) then begin
                oplot,[0,0],plotstr(pix).yrange,line=1,thick=thick,color=color
            endif 
            if i ne nev-1 and not keyword_set(noclick) then click
        endif else begin
            oplot,[data],[alt(gd)],$
              color=color,linestyle=linestyle,thick=thick,psym=psym,symsize=symsize
            if keyword_set(compare_version) and $
               not keyword_set(pdif) and not keyword_set(absdif) then begin
                oplot,[data2],[alt(gd2)],$
                  color=ccolor,linestyle=clinestyle,thick=cthick,$
                  psym=cpsym,symsize=csymsize
            endif
            if keyword_set(pdif) or keyword_set(absdif) then begin
                oplot,[0,0],plotstr(pix).yrange,line=1,thick=thick,color=color
            endif
        endelse
        if keyword_set(output) then begin
            printf,outlun,species,'  ',string(index.yyyymmdd(iev)),$
              string(index.hhmmss(iev)),string(index.lat(iev)),$
              string(index.lon(iev)),'  ',types(index.type_sat(iev))
            inrg = where(alt(gd) ge !y.crange(0) and $
                         alt(gd) le !y.crange(1), ninrg)
            for ialt = 0,ninrg-1 do begin
                printf,outlun,alt(gd(inrg(ialt))),data(inrg(ialt))
            endfor
            printf,outlun
        endif
    endif
endfor
print,'Finished.'
if keyword_set(output) then begin
    close,outlun & free_lun,outlun
endif
return
end





